// Main.java
//
// Informatics 43 Spring 2009
// Code Example: Serialization (Version 1)
//
// This program uses a manual approach to serialize Person data into a
// file and then deserialize it again, by converting the data to and
// from a simple textual format.

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Scanner;


public class Main
{
	public static void main(String[] args)
	{
		// First, we'll create an ArrayList of Person objects.
		ArrayList<Person> persons = new ArrayList<Person>();
		persons.add(new Person("Alex Thornton", "Continuing Lecturer"));
		persons.add(new Person("Norik D", "Lecture-Watcher and Student"));
		persons.add(new Person("Jonathan F", "Chief Executive Officer"));
		
		try
		{
			// Then, we'll save that data into a file called "persons.txt".
			saveData(persons, "persons.txt");
			
			// Finally, we'll read the data back in from the file and
			// print out each person's name and title, to see if it
			// loaded the same way we saved it.
			ArrayList<Person> persons2 = loadData("persons.txt");
			
			for (Person person : persons2)
			{
				System.out.println(person.getName() + ", " + person.getTitle());
			}
		}
		catch (IOException e)
		{
			// Quite often, we want to do something better with an exception
			// than to just catch it and print a stack trace.  This program
			// is mostly about exploring a new technique; we're the program's
			// only users and we'll understand the stack trace, so this is
			// sensible for our current purposes.  (Real customers shouldn't
			// be seeing stack traces, though, since they won't be able to
			// make sense out of them.)
			e.printStackTrace();
		}
	}
	

	// The saveData() method takes an ArrayList of Persons and a filename,
	// then saves data about all of the persons into a file with the given
	// filename.  The data is saved in a textual format, with names and
	// titles on separate lines.  If anything goes wrong while opening or
	// writing to the file, an IOException is thrown.
	private static void saveData(ArrayList<Person> persons, String filename)
	throws IOException
	{
		// FileWriters know how to write text into a file.  PrintWriters
		// layer some additional knowledge: how to write to a file in the
		// same way that we write to System.out, by calling methods like
		// print(), println(), and format().
		PrintWriter output = new PrintWriter(new FileWriter(filename));
		
		for (Person person : persons)
		{
			output.println(person.getName());
			output.println(person.getTitle());
		}
		
		output.close();
	}
	
	
	// The loadData() method takes a filename, reads the Person data
	// contained in that file, and returns an ArrayList of Persons
	// described by that data.  This method assumes that the file is
	// in the same format as one written by saveData().  If it fails
	// to open the file or read from it, an IOException is thrown.
	private static ArrayList<Person> loadData(String filename)
	throws IOException
	{
		// FileReaders know how read text from a file.  Scanners layer
		// some additional knowledge: how to read a whole line, a single
		// word, a single number, etc.
		Scanner input = new Scanner(new FileReader(filename));
		
		ArrayList<Person> persons = new ArrayList<Person>();
		
		while (input.hasNextLine())
		{
			String name = input.nextLine();
			String title = input.nextLine();
			
			persons.add(new Person(name, title));
		}
		
		input.close();
		return persons;
	}
}
