// Main.java
//
// Informatics 43 Spring 2009
// Code Example: Serialization (Version 2)
//
// This program uses Java's built-in serialization mechanism (via the
// ObjectInputStream and ObjectOutputStream classes) to save a Java
// object, along with all of the objects it refers to, into a file,
// then load that object back from the file.

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;


public class Main
{
	public static void main(String[] args)
	{
		// First, we'll create a Department and put some people into it.
		Department department = new Department();
		department.addPerson(new Person("Alex Thornton", "Continuing Lecturer"));
		department.addPerson(new Person("Norik D", "Lecture-Watcher and Student"));
		department.addPerson(new Person("Jonathan F", "Chief Executive Officer"));
		
		try
		{
			// Next, we'll save the department into a file using
			// serialization.  Since the file is not being saved in a 
			// text format, I'll use a different file extension (.dat)
			// instead of .txt.
			saveData(department, "department.dat");
			
			// Then, we'll load the department back from the file,
			// then print out information about all of its people,
			// to make sure that we got back the people we started
			// with.
			Department department2 = loadData("department.dat");
			
			for (Person person : department2.getPersons())
			{
				System.out.println(person.getName() + ", " + person.getTitle());
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	
	private static void saveData(Department department, String filename)
	throws IOException
	{
		// A FileOutputStream knows how to write bytes to a file.  An
		// ObjectOutputStream adds one ability: the ability to write
		// entire objects into an output stream, by serializing the
		// objects into a sequence of bytes.  Since our Department
		// objects are serializable, we can write a Department object
		// into an ObjectOutputStream by calling a method called
		// writeObject().
		
		ObjectOutputStream output =
			new ObjectOutputStream(new FileOutputStream(filename));
		
		output.writeObject(department);
		output.close();
	}
	

	private static Department loadData(String filename)
	throws IOException, ClassNotFoundException
	{
		// A FileInputStream knows how to read bytes from a file.  An
		// ObjectInputStream adds one ability: the ability to read
		// entire objects from an output stream, by deserializing
		// them.
		
		ObjectInputStream input =
			new ObjectInputStream(new FileInputStream(filename));
		
		Department department = (Department) input.readObject();
		
		return department;
	}
}
